@@ -1,9 +1,6 @@
 """Stubs for requests"""
 
-try:
-    from urllib3.connectionpool import HTTPConnection, VerifiedHTTPSConnection
-except ImportError:
-    from requests.packages.urllib3.connectionpool import HTTPConnection, VerifiedHTTPSConnection
+from urllib3.connection import HTTPConnection, VerifiedHTTPSConnection
 
 from ..stubs import VCRHTTPConnection, VCRHTTPSConnection
 
